<?php

namespace App\Livewire\Admin\Settings\System;

use Illuminate\Support\Facades\File;
use Illuminate\Support\Str;
use Livewire\Component;

class CacheManagementSettings extends Component
{
    public array $cacheSizes = [];

    public string $loadingType = '';

    public function mount()
    {
        if (! checkPermission('system_settings.view')) {
            $this->notify(['type' => 'danger', 'message' => t('access_denied_note')], true);

            return redirect(route('admin.dashboard'));
        }

        $this->calculateSizes();
    }

    public function calculateSizes(): void
    {
        $directories = [
            'framework' => storage_path('framework/cache'),
            'views'     => storage_path('framework/views'),
            'config'    => base_path('bootstrap/cache'),
            'routing'   => storage_path('framework/routes'),
            'logs'      => storage_path('logs'),
        ];

        $this->cacheSizes = array_map(
            fn ($path) => $this->getDirectorySize($path),
            $directories
        );
    }

    public function clearCache(string $type): void
    {
        $this->loadingType = $type;

        match ($type) {
            'framework' => clear_cache(),
            'views'     => clear_view(),
            'config'    => clear_config(),
            'routing'   => clear_route(),
            'logs'      => $this->clearLogFiles(),
            default     => throw new \InvalidArgumentException("Invalid cache type: {$type}"),
        };

        // Update only the cleared cache size
        $this->cacheSizes[$type] = $this->getDirectorySize($this->getDirectoryPath($type));

        $this->loadingType = '';

        $this->notify([
            'type'    => 'success',
            'message' => Str::headline($type) . t('cache_cleared_successfully'),
        ]);
    }

    private function getDirectoryPath(string $type): string
    {
        return match ($type) {
            'framework' => storage_path('framework/cache'),
            'views'     => storage_path('framework/views'),
            'config'    => base_path('bootstrap/cache'),
            'routing'   => storage_path('framework/routes'),
            'logs'      => storage_path('logs'),
            default     => throw new \InvalidArgumentException("Invalid cache type: {$type}"),
        };
    }

    private function getDirectorySize(string $path): string
    {
        if (! is_dir($path)) {
            return '0 B';
        }

        $size = iterator_count(
            new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator($path, \FilesystemIterator::SKIP_DOTS)
            )
        );

        return $this->formatSizeUnits($size);
    }

    private function clearLogFiles(): void
    {
        $path  = storage_path('logs');
        $files = File::glob("{$path}/[!whats]*.log");
        foreach ($files as $file) {
            File::delete($file);
        }
    }

    private function formatSizeUnits(int $bytes): string
    {
        return match (true) {
            $bytes >= 1_073_741_824 => number_format($bytes / 1_073_741_824, 2) . ' GB',
            $bytes >= 1_048_576     => number_format($bytes / 1_048_576, 2) . ' MB',
            $bytes >= 1_024         => number_format($bytes / 1_024, 2) . ' KB',
            $bytes > 1              => "{$bytes} bytes",
            $bytes === 1            => '1 byte',
            default                 => '0 B',
        };
    }

    public function render()
    {
        return view('livewire.admin.settings.system.cache-management-settings');
    }
}
